/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECMDLINEARGS_H
#define ECMDLINEARGS_H

#include "eglobal.h"

class ECmdLineOption;
class EPart;

#include <QList>
#include <QPointer>

class KCmdLineOptions;

/**
	* @brief Class to handle command line arguments and options.
	*
	* The class allows easy handling of command line arguments and options.
	* To add a provided option or argument, use
	* add(const QByteArray&, const QByteArray&, const QString&, const QByteArray&)
	* or add(ECmdLineOption*). To get information about the added options,
	* call init() first. Afterwards you can use isSet() to determine, if
	* an option is set or getOption() to get back the result of the option.
	* @note The options -v --version and -h --help are added and handled automatically.
	*
	* @code
	*	// Create an ECmdLineArgs object to handle command line options.
	*	ECmdLineArgs args;
	*	// Register the given arguments from command line
	* args.init(argc, argv);
	*	// register all possible options
	*	// e.g. an optional option, which requires an argument
	*	args.add("[user] <argument>", "u", "The user");
	*	// e.g. an needed argument
	*	args.add("+file", "", "A path to a file");
	*	// than where you want to parse your options:
	*	// check whether the option is given at the command line
	*	if(args.isSet("user")
	*	  // retrieve the argument of the option
	*	  QString username = args.getOption("user");
	* @endcode
	* @see ECmdLineOption
	*/
class E_CORE_EXPORT ECmdLineArgs
{
	public:
		/**
			* Specify an EPart with @p part, to let showHelp() and showVersion()
			* display the extended help of the part.
			*/
		ECmdLineArgs(EPart *part = NULL);
		virtual ~ECmdLineArgs();
		
		/**
		  * @brief Instantiate the given options.
			*
			* @return true if everything can be proceeded and false,
			*    if the application should exit. E.g. this can be the case,
			*    if the option -v or -h is chosen.
			*/
		bool init(int argc, char** argv);
		/**
			* @brief Add a new command line option to the list.
			*
			* This function creates a new ECmdLineOption object with
			* the specified content. To add an existent ECmdLineOption
			* use add(ECmdLineOption*).
			*/
		void add(const QByteArray &name,
						 const QByteArray &alias = QByteArray(),
						 const QString &description = QString(),
						 const QByteArray &defaultValue = QByteArray());
		/**
			* @brief Adds an existent ECmdLineOption to the option list.
			*
			* To create a new option, use 
			* add(const QByteArray&, const QByteArray&, const QString&, const QByteArray&).
			*/
		void add(ECmdLineOption *option);
		/**
			* @brief Checks wether the @p option is set.
			*
			* If @p option is set, it also set's the value of the @p option
			* to the first value of the occurence.
			* @p option is the name of the option without any special characters
			* like [], + or so (e.g. For "+[optionname] @p option should be optionname).
			* @todo improve handling of optional arguments and not optional arguments
			*/
		bool isSet(const QByteArray &option) const;
		/**
			* @brief Returns the value of @p option.
			*
			* It uses isSet() to set the value of @p option.
			* @p option is the name of the option without any special characters
			* like [], + or so (e.g. For "+[optionname] @p option should be optionname).
			*/
		QString getOption(const QByteArray &option) const;
		/**
			* @brief Adds all options to the given KCmdLineOptions @p option.
			*
			* If KDE is not enabled, this function does nothing.
			*/
		void addToKCmdLineOptions(KCmdLineOptions *option) const;
		
	protected:
		/**
			* @brief Shows the help.
			*
			* If you use ECmdLineArgs within your own application (not EPart based)
			* you should subclass it and reimplement this function along with showVersion().
			*/
		virtual void showHelp() const;
		/**
			* @brief Shows version information of Qt, KDE, LibEngsas and the EPart.
			*
			* If you use ECmdLineArgs within your own application (not EPart based)
			* you should subclass it and reimplement this function along with showHelp().
			*
			* @todo show LibEngsas version number
			*/
		virtual void showVersion() const;

	private:
		ECmdLineOption *findOption(const QByteArray &option) const;
		bool isSet(ECmdLineOption *option) const;
		
		EngSaS::CmdLineOptions options;
		int argc;
		char** argv;
		QPointer<EPart> part;
};

#endif // ECMDLINEARGS_H
