/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECMDLINEOPTION_H
#define ECMDLINEOPTION_H

#include "eglobal.h"

#include <QByteArray>
#include <QString>

class KCmdLineOptions;

/**
	* @brief Represents one command line argument or option.
	*
	* It is used by ECmdLineArgs to handle available options
	* and parse the given arguments.
  * @todo add convenience functions for the use of QStrings instead of QByteArrays
	*/
class E_CORE_EXPORT ECmdLineOption
{
	public:
		/**
			* To find out more about the parameters
			* and espacially about @p name, to add optional
			* options and or arguments, look at setName().
			* @param name see setName()
			* @param alias see setAlias()
			* @param description see setDescription()
			* @param defaultValue see setDefaultValue()
			*/
		ECmdLineOption(const QByteArray &name,
						 const QByteArray &alias = QByteArray(),
						 const QString &description = QString(),
						 const QByteArray &defaultValue = QByteArray());

		/**
			* @brief Sets the name of the option.
			*
			* It also analyze the name as following:
			* @li name \<argument> - The option with name "name" requires an argument
			* @li +name - The option is an required argument
			* @li [name] - The option is optional (combineable with the ones before)
			*/
		void setName(const QByteArray &name);
		QByteArray name() const;
		/**
			* @brief Returns true, if the option requires an argument.
			*
			* This is the case, if the option name is something like
			* @code
			*   setName("option <argument>");
			* @endcode
			*/
		bool requiresArgument() const;
		/**
			* @brief Returns true, if the option is an argument.
			*
			* This is the case, if the name is something like
			* @code
			*  setName("+option");
			* @endcode
			*/
		bool isArgument() const;
		/**
			* @brief Returns true, if the option or argument is optional.
			*
			* This is the case, if the option name contains [ and ].
			*/
		bool isOptional() const;
		/**
			* @brief The alias of this option.
			*
			* E.g. set it to "u" if the option is "user" and
			* should be accessible through "-u".
			*/
		void setAlias(const QByteArray &alias);
		QByteArray alias() const;
		/**
			* @brief The description of this option, which is used at the help.
			*/
		void setDescription(const QString &description = QString());
		QString description() const;
		/**
			* @brief The default vaule, if no value is given.
			*
			* The default default value is an empty QByteArray().
			*/
		void setDefaultValue(const QByteArray &defaultValue = QByteArray());
		QByteArray defaultValue() const;
		void setIsSet(bool status = true);
		/**
			* @brief Is true, if the option is set at the command line.
			*/
		bool isSet() const;
		void setValue(const QByteArray &value);
		/**
			* @brief Returns the value, which is specified via the command line.
			*
			* Or it returns an the defaultValue() if no value is given.
			*/
		QByteArray value() const;
		/**
			* @brief The part at the usage line in help.
			*/
		QString usage() const;
		/**
			* @brief The help line for this option in help.
			*/
		QString helpLine() const;
		/**
			* @brief Adds this option to the given KCmdLineOptions @p option.
			*
			* If KDE is not enabled, this function does nothing.
			*/
		void addToKCmdLineOptions(KCmdLineOptions *option) const;

	private:
		QByteArray myName, myAlias;
		QByteArray myDefaultValue, myValue;
		QString myDescription;
		bool myIsSet, myRequiresArgument;
		bool myIsArgument, myIsOptional;
};

#endif // ECMDLINEOPTION_H
