/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#ifndef EFILESERVICE_H
#define EFILESERVICE_H

#include "eobject.h"
#include "eglobal.h"

#include "efileinfo.h"

class EFileServicePlugin;
class ESettingsModelItem;

#include <QFileInfo>

/**
	* @brief Service which uses EFileServicePlugin's to handle files.
	* 
	* If you want to provide file handling in your application, use this class.
	* To let the user choose, which file service plugin he wants to use, append
	* the ESettingsModelItem from settings() to your list of settings. After the user
	* has chosen a service, just use this class to act on a file. Normally you walk through
	* these steps:
	* @li Get an EFileInfo of a file with add(). Further on, everything you want to do with that
	* 		file, use the returned EFileInfo.
	* @li To remember the added file, just store the unique identifier of the file, which
	* 		can be accessed with EFileInfo::id(). To get a new EFileInfo object for that
	*			file, call info() with that unique id.
	*/
class E_CORE_EXPORT EFileService : public EObject
{
	Q_OBJECT
	public:
		EFileService(QObject *parent = NULL);
		/**
			* Ignores the chosen file service at the settings and loads
			* the one given by @p serviceId.
			*/
		EFileService(QString serviceId, QObject *parent = NULL);
		
		/**
			* @brief A list with all EFileServicePlugins.
			*/
		static QList<EFileServicePlugin*> services();
		/**
			* @brief The settings page for the EFileService module.
			* 
			* Does also contain the page for the currently selected EFileServicePlugin.
			*/
		static ESettingsModelItem* settings();
		
		/**
			* @brief Returns true, if a service is loaded and false if not.
			*/
		bool serviceLoaded() const;
		
		EFileInfo info(QString uniqueId) const;
		/**
			* @brief Adds file described by @p info to the handled files.
			* 
			* If @p create is set to true, the file should be created,
			* if it does not exists already.
			* 
			* Adding as is can mean, add a local file to an repository
			* or to a remote file system or something else.
			* In the case of the EFileSystemPlugin, this functions only
			* converts the QFileInfo into an EFileInfo, because the files
			* are the physically the same.
			*/
		EFileInfo add(QFileInfo info, bool create = false) const;
		/**
			* @brief Moves the file from @p source to @p target.
			* 
			* @p target should be an absolute file path.
			* @see copy() to copy @p source.
			*/
		bool move(EFileInfo source, QString target) const;
		/**
			* @brief Copies the file from @p source to @p target.
			* 
			* @p target should be an absolute file path.
			* @see move() to move @p source.
			*/
		bool copy(EFileInfo source, QString target) const;
		bool deleteFile(EFileInfo file) const;
		bool open(EFileInfo file);
		/**
			* @brief Switches all files from current service to @p newServiceId.
			* 
			* @todo Must be implemented!
			*/
		bool switchService(QString newServiceId);
		
	private:
		EFileServicePlugin *loadService(QString serviceId);
		
		EFileServicePlugin *service;
};

#endif // EFILESERVICE_H
