/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

/**
	* @mainpage LibEngsas
	*
	* @section sec1 About
	*
	* LibEngsas provides improved Qt classes, to make common tasks easier.
	* To include it in your applications, use FIND_PACKAGE(LibEngsas) in your CMakeLists.txt. The Find-Module should be installed,
	* if this library is build with BUILD_DEV=ON (Or the -dev package is installed). For further details on the build process, see @ref build-process.
	*
	* @subsection sec11 Most important differences
	*
	* The biggest advantage of this library is provided by EWidget, EObject, EDialog and EPart.
	* Therfore you should read the documentation of this class very carefully to get the whole benefit.
	* Another improvement is the support of QIcon::fromTheme for some GUI Elements like EAction (replacement for QAction)
	* and EPushButton (replacement for QPushButton).
	*
	* Another important part is the SQL handling. LibEngsas provides classes to create, update and
	* backup databases. Therefore take a look at the ESqlInit, ESqlTable and ESqlField classes. Also
	* the class ESqlDatabase may be interesting, because it provides a more easier way to reuse
	* an existant database connection and therefore it made transaction handling more easy.
	*
	* Also some common settings are provided. These includes MimeType settings and language settings.
	* They are used by all applications, which uses this library with class like EOpenFileExternal or the epartexe
	* to launch an EPart.
	* 
	* @subsection sec12 Advantages for developers
	* 
	* LibEngsas provides @ref libengsas-testing for developers. This library includes some classes to make testing more
	* easy. For example the ModelTest class from Qt Labs is included. It also provides the thread CloseModalWidgetThread to handle
	* model dialogs during tests. For further details please read @ref libengsas-testing.
	* 
	*
	* @section sec2 Developing
	*
	* @subsection sec21 Prepare Environment
	*
	*
	* @subsection sec22 Contributing Work
	*
	* @author Joachim Langenbach
	* 
	* 
	* 
	* \page build-process Build Process
	* LibEngsas uses CMake to configure the build environment. Therefore we prefer out of source builds.
	* For a standard build, just do the following steps, below the libengsas root directory:
	* @code
	*   mkdir build
	*   cd build
	*   cmake ../
	*   make
	*   sudo make install
	* @endcode 
	* 
	* Per default, LibEngsas tries to build the KDE verison, but disables KDE automatically,
	* if the KDE Library is not found. In this standard mode, it builds the complete library
	* and installs it with same name and same locations as the non KDE version. If you want
	* to provide the KDE and non KDE version on same system, you should set COMPATIBLE_WITH_NON_KDE=ON.
	* With this option enabled, the KDE build only builds the KDE specific parts and installs them in
	* different locations or with different names (e.g. epartexe is named epartexe-kde and libraries are
	* installed into a kde subdirectory).
	* To install LibEngsas into /usr and not into /usr/local, which is the default, specify the
	* CMAKE_INSTALL_PREFIX=/usr.
	* 
	*  @note In both situations, you may find it useful to add a libengsas.conf file into /etc/ld.so,conf.d
	*        to point ld to the path, where the libraries are installed.
    *
    * @section secAndroidBuild Android
    *
    * @code
    *   "c:\Program Files (x86)\CMake 2.8\bin\cmake.exe" -DCMAKE_TOOLCHAIN_FILE="c:\libengsas\cmake\android.toolchain.cmake" \
	*       -DQT_QMAKE_EXECUTABLE="c:\Qt\5.1.1\android_armv7\bin\qmake.exe" -DCMAKE_PREFIX_PATH="c:\Qt\5.1.1\android_armv7" \
	*       -G "MinGW Makefiles" -DANDROID_NDK="c:\android\android-ndk-r9b-windows-x86_64\android-ndk-r9b" \
	*       -DBINDINGS_QT5=ON -DL10N_ALL=OFF -DWITH_FILE_SERVICE_PLUGINS_ALL=OFF -DWITH_SPREADSHEET_PROVIDER_PLUGINS_CSV=OFF \
	*       -DBUILD_TESTING=OFF -DWITH_EXAMPLES=OFF -DWITH_API_DOCS=OFF ../../
    * @endcode
	* 
	* 
  * @author Joachim Langenbach
  */

#ifndef EGLOBAL_H
#define EGLOBAL_H

#ifdef E_BUILD_CORE_LIB
	#define E_CORE_EXPORT Q_DECL_EXPORT
#else
  #define E_CORE_EXPORT Q_DECL_IMPORT
#endif

#include <QObject>

class ECmdLineOption;
class EAboutPerson;
class ESetupWizardObject;
class EModuleListWidgetItem;
class ESpreadsheet;

/**
	* @brief Some definitions, used along LibEngsas.
	*/
namespace EngSaS {
	/**
		* @brief Used to define the filters for file endings should be for loading or saving.
		*/
	enum FileFilter{
		Load,	/**< filters useable to load files */
		Save	/**< filters useable to save files */
	};

	Q_FLAGS(AvailableActions)
	/**
		* @brief Used to identify enabled standard actions of EMainWindow for active EWidget.
		* The explizit definition must be done, because otherwise the flags are not or combinable.
		*/
	enum AvailableAction
	{
		FileNew 		= 0x0001,	/**< The New entry in file menu */
		FileOpen 		= 0x0002,	/**< The entry Open in file menu */
		FileSave 		= 0x0004,	/**< The Save entry in file menu. SaveAs is also turned on, because a file selection is needed. */
		FileSaveAs 	= 0x0008,	/**< The Save As entry in file menu (devided from FileSave, because it is used as Export function too */
		FilePrint 	= 0x0016,	/**< Print entry in file menu */
		EditUndo 		= 0x0032,	/**< Undo entry in Edit menu */
		EditRedo 		= 0x0064,	/**< Redo entry in Edit menu */
		EditCut 		= 0x0128,	/**< Cut entry in Edit menu */
		EditCopy 		= 0x0256,	/**< Copy entry in Edit menu */
		EditPaste 	= 0x0512,	/**< Pase entry in Edit menu */
		ViewReload 	= 0x1024	/**< Reload entry in View menu */
	};
	Q_DECLARE_FLAGS(AvailableActions, AvailableAction)
	Q_ENUMS(AvailableAction)

	/**
	* @brief A type to replacce QList<ECmdLineOption*> with a shorter name.
	* @todo Must hte list contains pointers instead of objects?
	*/
	typedef QList<ECmdLineOption*> CmdLineOptions;
	/**
		* @brief A type to replace QList<EAboutPerson*> with a shorter name.
		*/
	typedef QList<EAboutPerson*> AboutPersons;
	/**
		* @brief A type to replace QList<ESetupWizardObject*> with a shorter name.
		*/
	typedef QList<ESetupWizardObject*> SetupWizardObjects;
	/**
		* @brief A type to replace QList<EModuleListWidgetItem*> with a shorter name.
		*/
	typedef QList<EModuleListWidgetItem*> ModuleListWidgetItems;
	/**
		* @brief A type to replace QList<ESpreadsheet*> with a shorter name.
		*/
	typedef QList<ESpreadsheet*> Spreadsheets;
	
	/**
		* @brief Some definitions related to SQL subsystem.
		*/
	namespace SQL {
		/**
			* @brief Choose the type of the used database.
			*/
    enum DatabaseType {
			Other,	/**< All SQL Databases, excluding SQLite */
			SQLite	/**< SQLite Database */
		};
	};
};

Q_DECLARE_OPERATORS_FOR_FLAGS(EngSaS::AvailableActions)

#endif // EGLOBAL_H
