/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EPLUGINLOADER_H
#define EPLUGINLOADER_H

#include "eglobal.h"
#include "emodulelistwidgetitem.h"

#include <QPluginLoader>
#include <QFileInfo>
#include <QDir>
#include <QObject>

/**
	* @brief Provides functions to load Qt plugins properly.
	*
	* To load your specific plugin type, use
	* @code
	* // to load the first matching name plugin:
	* YourPluginClass *plugin = EPluginLoader<YourPluginClass>::loadPlugin("name", "searchdir");
	* // to load all plugins:
	* QList<EModuleListWidgetitem*> log;
	* QList<YourPluginClass*> plugins = EPluginLoader<YourPluginClass>::loadPlugins("searchdir", log);
	* @endcode
	*
	* If you want to define the search dir not everytime, do something like this:
	* @code
	* #ifndef CUSTOMPLUGINLOADER_H
	* #define CUSTOMPLUGINLOADER_H
	* 
	* #include <epluginloader.h>
	*
	* template <class PluginType>
	* class CustomPluginLoader {
	*   public:
	*     static QList<PluginType*> loadPlugins(QList<EModuleListWidgetitem*> &log);
	*     static PluginType* loadPlugin(QString name, EModuleListWidgetitem* item = NULL);
	* };
	* 
	* template <class PluginType>
	* QList<PluginType*> CustomPluginLoader<PluginType>::loadPlugins(QList<EModuleListWidgetitem*> &log)
	* {
	*   return EPluginLoader<PluginType>::loadPlugins("YourDir", log);
	* }
	* 
	* template <class PluginType>
	* PluginType* CustomPluginLoader<PluginType>::loadPlugin(QString name, QString &error)
	* {
	*   return EPluginLoader<PluginType>::loadPlugin(name, "YourDir", log);
	* }
	* 
	* #endif // CUSTOMPLUGINLOADER_H
	* @endcode
	*
	* @note Template classes must be implemented in the header file.
	*    Otherwise, undefined reference errors appear.
	*/
template <class PluginType> class EPluginLoader
{
	public:
		/**
			* @brief Loads all plugins of type PluginType in a directory.
			*
			* The searched directory is @p searchPath. The information
			* about the loaded or failed plugins are stored in the @p log.
			*
			* @see loadPlugin() to load the first plugin with a specified name.
			*/
		static QList<PluginType*> loadPlugins(QString searchPath, EngSaS::ModuleListWidgetItems &log);
		/**
			* @brief Loads the first plugin matching PluginType and @p name in a directory.
			*
			* The searched directory is @p searchPath. The information
			* about the loaded or failed plugin is stored in the @p item.
			*
			* @see loadPlugins() to load all matching plugins.
			*/
		static PluginType *loadPlugin(QString name, QString searchPath, EModuleListWidgetItem *item = NULL);

		/**
			* @brief Used internally to really load a plugin.
			*
			* Normally you won't use this function directly,
			* use loadPlugin() or loadPlugins() instead.
			*/
		PluginType *doLoadPlugin(QString absFilePath, EModuleListWidgetItem *item = NULL);
};

template <class PluginType>
QList< PluginType* > EPluginLoader<PluginType>::loadPlugins(QString searchPath, EngSaS::ModuleListWidgetItems& log)
{
	QList<PluginType*> result;

	QDir dir(searchPath);
	if ( !dir.exists() || !dir.isReadable())
		return result;
	
	EPluginLoader<PluginType> mLoader;
	QStringList files = dir.entryList(QDir::Files | QDir::Readable);
	foreach ( QString file, files ) {
		// handle symlinks
		QFileInfo info(dir.filePath(file));
		if(info.isSymLink()){
			// check whether target is also in the list to avoid double loading
			bool doubleEntry = false;
			foreach(QString compareFile, files){
				if(compareFile == file)
					continue;
				if(info.symLinkTarget() == dir.filePath(compareFile)){
					doubleEntry = true;
					break;
				}
			}
			if(doubleEntry)
				continue;
		}
		
		EModuleListWidgetItem *item = new EModuleListWidgetItem();
		log << item;
		PluginType *plugin = mLoader.doLoadPlugin(dir.filePath ( file ), item);
		if(plugin)
			result << plugin;
	}
	return result;
}

template <class PluginType>
PluginType* EPluginLoader<PluginType>::loadPlugin(QString name, QString searchPath, EModuleListWidgetItem* item)
{
	if(name.isEmpty())
		return NULL;

	QDir dir(searchPath);
	if ( !dir.exists() || !dir.isReadable()){
		if(item)
			item->setErrorMessage(QObject::tr("Directory does not exists or is not readable:\n%1").arg(dir.absolutePath()));
		return NULL;
	}

	EPluginLoader<PluginType> mLoader;
	QStringList filter;
	filter << QString("*%1*").arg(name);
	QStringList files = dir.entryList (filter, QDir::Files | QDir::Readable);

	if(files.size() < 1){
		if(item){
			item->setErrorMessage(
				QObject::tr("%1 not found at \n%2\n\nFilters: %3\n")
					.arg(name)
					.arg(dir.absolutePath())
					.arg(filter.join(", "))
			);
		}
	}

	foreach ( QString file, files ) {
		PluginType *plugin = mLoader.doLoadPlugin(dir.filePath ( file ), item);
		if(plugin)
			return plugin;
	}
	return NULL;
}

template <class PluginType>
PluginType* EPluginLoader<PluginType>::doLoadPlugin(QString absFilePath, EModuleListWidgetItem* item)
{
	QFileInfo file(absFilePath);
	if(item)
		item->setFile(file.absoluteFilePath());
	if(!file.exists() || !file.isReadable()){
		if(item)
			item->setErrorMessage(QObject::tr( "File does not exists or is not readable:\n%1").arg(file.absoluteFilePath()));
		return NULL;
	}

	if(!QLibrary::isLibrary(file.absoluteFilePath())){
		if(item)
			item->setErrorMessage(QObject::tr("%1 is no library file").arg(file.absoluteFilePath()));
		return NULL;
	}

	QPluginLoader loader ( file.absoluteFilePath() );
	QObject *plugin = loader.instance();
	if ( !plugin ){
		if(item)
			item->setErrorMessage(
				QObject::tr( "While loading %1 an error occured:\n%2\n" )
					.arg ( file.absoluteFilePath() ).arg(loader.errorString())
			);
		return NULL;
	}

	PluginType *newPlugin = qobject_cast<PluginType *> ( plugin );
	if ( !newPlugin ){
		if(item)
			item->setErrorMessage(QObject::tr("Could not load %1!\n(Plugin is not of specified type!)").arg(file.absoluteFilePath()));
		return NULL;
	}

	if(item){
		item->setStatus(true);
		QObject *object = dynamic_cast<QObject*>(newPlugin);
		if(object)
			item->extractFromPlugin(object);
	}
	return newPlugin;
}

#endif // EPLUGINLOADER_H
