/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#ifndef EWIDGET_H
#define EWIDGET_H

#include "eglobal.h"
#include <QWidget>

#include "eobject.h"

class ESettingsWidget;

class QMenu;
class QAction;
class QToolBar;
class QDockWidget;
class QFile;

#include <QPointer>

/**
	* @brief EWidget is a base class for common widgets.
	*
	* It can handle all standard functionality which may be used.
	* If you want to cut of the by passing of signals, just reimplment the
	* virtual byPass() slot of the signal.
	* @see EObject
	* @see EDialog
	*/
class E_CORE_EXPORT EWidget : public QWidget
{
	Q_OBJECT

	public:
		inline EWidget ( QWidget *parent = 0, Qt::WindowFlags flags = 0 ) : QWidget(parent, flags) {}
		/**
			* Call close here, to get the close() function called in every possible case.
			*/
		virtual ~EWidget();
		/**
			* overloads QWidget::setWindowTitle() and prepends the application short name.
			* this is according to the Microsoft window guide line and also
			* common behaviour.
			*/
		void setWindowTitle ( const QString &title );
		/**
			* @brief Connects all standard signals.
			* @see EObject::connectByPass(QObject*)
			* @see EDialog::connectByPass(QObject*)
			*/
		void connectByPass(QObject *object){ EObject::connectByPass(this, object); }

	public slots:
		/**
			* @brief Will be called by EMainWindow::setCentralWidget() to enable and disable standard actions.
			*
			* @see EMainWindow::AvailableAction
			*/
		virtual EngSaS::AvailableActions enableActions(){ return 0; }
		/**
			* Severall filters can be seperated by ;; like
			* @code
			*   return tr("Text files (*.txt);;XML files (*.xml)");
			* @endcode
			* @return all filters for file prefixes, for @p reason.
			* @see EngSaS::FileFilter for existing cases
			* @see EFileDialog
			*/
		virtual QString fileFilters(EngSaS::FileFilter reason);
		/**
			* Try load @p file. You do not need to take care for opening or closing,
			* because the @p file is already opened for reading and will be closed afterwards.
			* Use @p error to describe the error for the user.
			* @return true if loading was successful, otherwise return false
			*/
		virtual bool load(QFile *file, QString &error);
		/**
			* Try to save content to @p file. Take care of the chosen file ending to allow exporting.
			* You do not need to take care for opening or closing,
			* because the @p file is already opened for writing and will be closed afterwards.
			* Use @p error to describe the error for the user.
			* @return true if successful, otherwise return false
			*/
		virtual bool save(QFile *file, QString &error);
		/**
			* Create new empty document
			* Use @p error to describe the error for the user.
			*/
		virtual bool fileNew(QString &error);
		/**
			* Print content
			* Use @p error to describe the error for the user.
			*/
		virtual bool print(QString &error);
		/**
			* undo last
			* Use @p error to describe the error for the user.
			*/
		virtual bool undo(QString &error);
		/**
			* redo last
			* Use @p error to describe the error for the user.
			*/
		virtual bool redo(QString &error);
		/**
			* cut selected
			* Use @p error to describe the error for the user.
			*/
		virtual  bool cut(QString &error);
		/**
			* copy selected
			* Use @p error to describe the error for the user.
			*/
		virtual bool copy(QString &error);
		/**
			* paste selected
			* Use @p error to describe the error for the user.
			*/
		virtual bool paste(QString &error);
		/**
			* reload content
			* Use @p error to describe the error for the user.
			*/
		virtual bool reload(QString &error);
		/**
			* @brief Used to reload settings the settings.
			*
			* This function is connected to doReloadSettings() and should be used
			* to actually do the reload of the settings. To indicate, that the settings
			* should be reloaded, emit reloadSettingsRequest().
			* Use @p error to describe the error for the user. And don't forget to
			* call the base implementation or emit doReload() yourself.
			*
			* @see doReloadSettings()
			* @see reloadSettingsRequest()
			* @see EObject::reloadSettings()
			* @see EDialog::reloadSettings()
			*/
		virtual inline void reloadSettings(){ emit doReloadSettings(); }

	protected slots:
		/**
			* @see error() if a child emits this signal, this class emits error()
			* (it passes the childs signal upwards)
			*/
		virtual inline void errorByPass(const QString &message){ emit error(message); }
		/**
			* @see warning() passes the emitted signals from it's child forward
			*/
		virtual inline void warningByPass(const QString &message){ emit warning(message); }
		/**
			* @see information() passes the emitted signals from it's child forward
			*/
		virtual inline void informationByPass(const QString &message){ emit information(message); }
		/**
			* Passes the reloadEnableActions() signals from it's childs forward.
			*/
		virtual inline void reloadEnableActionsByPass(){ emit reloadEnableActions(); }
		/**
			* @see addMenu() passes the emitted signals from it's child forward
			*/
		virtual inline void addMenuByPass(QMenu* newMenu, const QString &parent, QString before = QString()){ emit addMenu(newMenu, parent, before); }
		/**
			* @see addAction() passes the emitted signals from it's child forward
			*/
		virtual inline void addActionByPass(QAction* newAction, const QString &parent, QString before = QString()){ emit addAction(newAction, parent, before); }
		/**
			* @see addToolBar() passes the emitted signals from it's child forward
			*/
		virtual inline void addToolBarByPass(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea){ emit addToolBar(newToolBar, area); }
		/**
			* @see addNewDockWidget() passes the emitted signals from it's child forward
			*/
		virtual inline void addNewDockWidgetByPass(QDockWidget *widget, const Qt::DockWidgetArea &area){ emit addNewDockWidget(widget, area); }
		/**
			* @see setCentralWidget() passes the emitted signals from it's child forward
			*/
		virtual inline void setCentralWidgetByPass(QWidget *centralWidget){ emit setCentralWidget(centralWidget); }
		/**
			* @see showSettings() passes the emitted signals from it's child forward
			*/
		virtual inline void showSettingsByPass(){ emit showSettings(); }
		/**
			* @see showHelp() passes the emitted signals from it's child forward
			*/
		virtual inline void showHelpByPass(const QString &page){ emit showHelp(page); }
		/**
			* @see showStatusWidget() passes the emitted signals from it's child forward
			*/
		virtual inline void showStatusWidgetByPass(QWidget* widget){ emit showStatusWidget(widget); }
		/**
			* @see reloadSettingsRequest() passes the emitted signals from it's child forward.
			* @see reloadSettings()
			* @see reloadSettingsByPass()
			* @see doReloadSettings()
			*/
		virtual inline void reloadSettingsRequestByPass(){ emit reloadSettingsRequest(); }

	protected:
		/**
			* @brief Adds an dock widget via addNewDockWidget() and adds @p widget to the list of managed dock widgets.
			*
			* If @p actionLabel (and also actionParent) is set, an EAction is created and added to parent @p actionParent to show and
			* hide the dock widget. This action is also managed and therefore deleted along with the dock widget itself.
			*
			* The benefit is, that all the dock widgets on this list are deleted automatically, if the widget
			* it self is deleted.
			*/
		void addManagedDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area,
															QString actionLabel = QString(), QString actionParent = QString());

	private:
		QList<QPointer<QDockWidget> > dockWidgets;
		QList<QPointer<QAction> > dockActions;

	signals:
		/**
			* @see EObject::error()
			* @see warning()
			* @see information()
			* @see errorByPass()
			*/
		void error(const QString &message);
		/**
			* @see EObject::warning()
			* @see error()
			* @see information()
			* @see warningByPass()
			*/
		void warning(const QString &message);
		/**
			* @see EObject::information()
			* @see error()
			* @see warning()
			* @see informationByPass()
			*/
		void information(const QString &message);
		/**
			* @brief Request reloading of enabled actions in EMainWindow.
			*/
		void reloadEnableActions();
		/**
			* @see EObject::addMenu()
			* @see addAction() to add an action
			* @see addToolBar()
			* @see addMenuByPass()
			*/
		void addMenu(QMenu* newMenu, const QString &parent, QString before = QString());
		/**
			* @see EObject::addAction()
			* @see addMenu()
			* @see addToolBar()
			* @see addActionByPass()
			*/
		void addAction(QAction* newAction, const QString &parent, QString before = QString());
		/**
			* @see EObject::addToolBar()
			* @see addMenu()
			* @see addAction()
			* @see addToolBarByPass()
			*/
		void addToolBar(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea);
		/**
			* @brief To add managed dock widgets, use addManagedDockWidget() instead.
			*
			* @see EObject::addNewDockWidget()
			* @see setCentralWidget()
			* @see addNewDockWidgetByPass()
			*/
		void addNewDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area);
		/**
			* @see EObject::setCentralWidget()
			* @see addNewDockWidget()
			* @see setCentralWidget()
			*/
		void setCentralWidget(QWidget *centralWidget);
		/**
			* @see EObject::showSettings()
			* @see showSettingsByPass()
			*/
		void showSettings();
		/**
			* @see EObject::showHelp()
			* @see showHelpByPass()
			*/
		void showHelp(const QString &page);
		/**
			* @see EObject::showStatusWidget()
			* @see showStatusWidgetByPass()
			*/
		void showStatusWidget(QWidget* widget);
		/**
			* @brief Indicates, that a reload of the settings is needed.
			*
			* Therefore this signal is used to bring the reload message upwards to the main window
			* or whatever.
			* @see reloadSettingsRequestByPass()
			* @see doReloadSettings()
			* @see reloadSettings()
			* @see EDialog::reloadSettingsRequest()
			* @see EObject::reloadSettingsRequest()
			*/
		void reloadSettingsRequest();
		/**
			* @brief Emitted to bring the reload request back to the farest child.
			* @see reloadSettings()
			* @see reloadSettingsRequest()
			* @see EDialog::doReloadSettings()
			* @see EObject::doReloadSettings()
			*/
		void doReloadSettings();
};

#endif // EWIDGET_H
