/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EXMLITEM_H
#define EXMLITEM_H

#include "eglobal.h"
#include <QObject>

class EXmlParserItem;

#include <QVariant>
#include <QDomElement>

/**
  * @brief An EXmlItem is used to handle XML tags and it's atrributes and or child tags.
	*
	* It must be registered to the model with help of EXmlModel::registerParser(),
	* so that the model knows, which tagname can be parsed by which EXmlItem.
	* The function used as a callback must be a static public function and must
	* return a pointer to an EXmlItem. The function shown below should do it in
	* nearly all cases:
	* @code
	*   static inline EXmlItem *create(EXmlModel *model){ return new MyClass(); }
	* @endcode
	* The pointer to the currently used EXmlModel is provided, by the EXmlParserItem
	* created by EXmlModel::registerParser() automatically. Therefore you may use this
	* pointer within your item or creation method to access the model.
	*
	* @see EDependencyItem as an example implementation
	*/
class E_CORE_EXPORT EXmlItem : public QObject
{
	Q_OBJECT
	public:
		EXmlItem(QString tagName, EXmlItem *parent = 0);
		virtual ~EXmlItem();

		/**
			* Is called from the Model or the parent item, to load the content of @p element.
			* Therefore this function must also create child EXmlItem's, if child tags are found,
			* which can not be handled by it's own. For this purpose the mapping @p parser
			* is given. The key of this map is the tagName which belongs to the parser item.
			* If the found tag is not in the list, return false. If it is found, call the
			* EXmlParseritem::load(QDomElement, EXmlItem *, QMap<QString, EXmlParserItem*>)
			* function to process underlaying data. But create a new QDomElement as a  child.
			*
			* @note If adding childs with addChild(), set emitLayoutChanges to false, to avoid
			*       emitting not needed layout changed signals.
			*
			* @see EDependencyItem::load() as an implementation example
			*/
		virtual bool load(const QDomElement element, QMap<QString, EXmlParserItem*> parser) = 0;
		/**
			* This function should convert the current handled data back into a QDomElement.
			* The given @p useElem should be used to store the own data into. Your childs
			* should be appended to childs of @p useElem. These can be done by calling
			* this base implementation after processing your part.
			* @return false, if @p useElem is a null element or something other went wrong.
			* @see EDependencyItem::save() for an example implementation
			*/
		virtual bool save(QDomElement &useElem);

		int rowCount() const;
		/**
			* @brief used by EXmlModel::data() to retrieve the @p role data of this item.
			*
			* Implement this function to return data to the view
			*
			* @see EDependencyItem::data()
			*/
		virtual QVariant data(int column, int role = Qt::DisplayRole) const = 0;
		/**
			* @brief Called by EXmlModel::setData() to set the @p role data for this item to @p value.
			*
			* The default implementation returns just false. So implement it, if you want to
			* provide editable items.
			*/
		virtual bool setData(int column, QVariant value, int role = Qt::EditRole);

		/**
		 * @brief Do not forget to call the base class implementation, when reimplementing this function.
		 */
		virtual void setParent(EXmlItem *parent);
		EXmlItem *parent() const;
		/**
			* @brief Adds @p item as a child to this item at position @p row.
			*
			* It also sets the parent of @p item to this object.
			*
			* Per default, also the signals layoutAboutToBeChanged() and layoutChanged()
			* are emitted. To avoid those signals beeing emited, set @p emitLayoutChanges to false.
			* It is useful to set it to false, if you want to insert a large amount of items.
			* Therefore the changes shoudl not be emited within the load() function.
			*/
		virtual bool addChild(int row, EXmlItem *item, bool emitInsertRows = true);
		/**
		  * @brief Removes and deletes the child at @p row.
		  */
		virtual bool removeChild(int row, bool emitRemoveRows = true);
		EXmlItem *child(int row) const;
		/**
			* @brief Returns the row of the @p child.
			* 
			* If the child is not found or any other error occurs
			* -1 is returned.
			*/
		int row(EXmlItem *child) const;
		void setTagName(QString tagName);
		/**
			* @brief Return the tag name (in lowercase), which is handled by this EXmlItem.
			*
			** @note tagName() is turned into lowercase.
			*/
		QString tagName() const;
		
		/**
			* @brief Returns a pointer to a new element, copied from this one.
			*
			* To copy all standard stuff, use the copyBase(EXmlItem*) function.
			* So implement this function in all base classes like this:
			* @code
			* YourClass *myCopy = new YourClass();
			* copyBase(myCopy);
			* // now set your custom values. e.g.:
			* myCopy->setCustomValue(customValue());
			* return myCopy;
			* @endcode
			* Remember, that copyBase() does copy recursivly. That means, it adds
			* copies of all children to the copy.
			*/
		virtual EXmlItem *copy() const = 0;
			
	signals:
		void changed(EXmlItem *item);
		void rowsAboutToBeInserted(EXmlItem *parent, int start, int end);
		void rowsInserted();
		void rowsAboutToBeRemoved(EXmlItem *parent, int start, int end);
		void rowsRemoved();

			
	protected slots:
		/**
		  * @brief Just forwards the changed signal.
		  *
		  * To interupt the forwarding, reimplement this function.
		  */
		inline virtual void changedByPass(EXmlItem *item){ emit changed(item); }
		/**
			* @brief Just forwards the rowsAboutToBeInserted() signal.
			*
			* To interupt the forwarding, reimplement this function.
			*/
		inline virtual void rowsAboutToBeInsertedByPass(EXmlItem *parent, int start, int end){ emit rowsAboutToBeInserted(parent, start, end); }
		/**
			* @brief Just forwards the rowsInserted() signal.
			*
			* To interupt the forwarding, reimplement this function.
			*/
		inline virtual void rowsInsertedByPass(){ emit rowsInserted(); }
		/**
			* @brief Just forwards the rowsAboutToBeRemoved() signal.
			*
			* To interupt the forwarding, reimplement this function.
			*/
		inline virtual void rowsAboutToBeRemovedByPass(EXmlItem *parent, int start, int end){ emit rowsAboutToBeRemoved(parent, start, end); }
		/**
			* @brief Just forwards the rowsRemoved() signal.
			*
			* To interupt the forwarding, reimplement this function.
			*/
		inline virtual void rowsRemovedByPass(){ emit rowsRemoved(); }
		
	protected:
		/**
			* @brief Copies all the standard stuff into @p target.
			*
			* Call this function from copy().
			* Remember that this function copies recursivly. That means, it adds
			* copies of all children to @p target.
			*
			* When subclassing, it can be useful to reimplement this function, to 
			* allow copy & paste code from copy() help at the subclasses of your
			* subclasses. In this case, you should remamber to call the base implementation
			* at the appropriate place:
			* @code
			* EXmlItem::copyBase(target);
			* // copy your base stuff
			* @endcode
			*/
		virtual void copyBase(EXmlItem *target) const;

		
	private:
		EXmlItem *myParent;
		QList<EXmlItem*> childs;
		QString myTagName;
};

Q_DECLARE_METATYPE(EXmlItem*)

#endif // EXMLITEM_H
